// MokujIN, a string multiplier, written by Kaze, 2012-Nov-07, revision 3+.
// Free download at: www.sanmayce.com/Downloads/MokujIN.zip

// Revision 3:
// Computing 2^^1048576 took 0,454 seconds with '/TURBO' with Intel v12.1 on T7500 2200MHz.
// Computing 2^^1048576 took 1,856 seconds without '/TURBO' with Intel v12.1 on T7500 2200MHz.
// Computing 2^^1048576 took 0,426 seconds with '/TURBO' with Microsoft v16 on T7500 2200MHz.
// Computing 2^^1048576 took 1,678 seconds without '/TURBO' with Microsoft v16 on T7500 2200MHz.

// On August 23rd, 2008, a UCLA computer in the GIMPS PrimeNet network discovered the 45th (biggest so far (2010-July)) known Mersenne prime,
// 2^43,112,609-1, a mammoth 12,978,189 digit number!
// Now, who can wait for 'MokujIN.exe 2 43112609 /turbo' to complete?!

/*
D:\_KAZE\MokujIN>MokujIN_Microsoft_32-bit_Version_16.exe /help
MokujIN, Multiplication of INtegers, written by Kaze, 2012-Nov-06, revision 3.
Usage: MokujIN [Number Power [/turbo]]
Note1: Power is signed 32bit integer i.e. up to 2^31-1=2,147,483,647.
Note2: Multiplicand or/and Multiplier cannot exceed 12978189 digits.
Note3: TURBO regime is several times (only) faster.
Example1: D:\_KAZE\MokujIN>MokujIN.exe 18446744073709551616 2
340282366920938463463374607431768211456
Example2: D:\_KAZE\MokujIN>MokujIN.exe
Multiplicand: 18446744073709551616
Multiplier  : 18446744073709551616
Result      : 340282366920938463463374607431768211456

D:\_KAZE\MokujIN>
*/

#include <stdio.h>
#include <time.h>
#define memory_size 12978189 // equals MAX(Multiplicand,Multiplier)

unsigned char* MUL(unsigned char* Result, unsigned char* Multiplicand, unsigned char* Multiplier)
{
/*
'QuickBASIC was/is an excellent environment/compiler, eh.
FUNCTION MUL$ (Multiplicand$, Multiplier$)
'DimSum = CandLength + ErLength
'REDIM Result%(1 TO DimSum)
FOR SF = 1 TO CandLength 
FOR QB = 1 TO ErLength
	CarryFlag = 0
	Cycle = QB - 1 + SF
	Tiller = Cand%(SF) * Er%(QB)
	Result%(Cycle) = Tiller MOD 10 + Result%(Cycle)
	IF Result%(Cycle) >= 10 THEN
		Result%(Cycle) = Result%(Cycle) - 10
		CarryFlag = 1
	END IF
	NextNumPos = Cycle + 1
	Result%(NextNumPos) = Result%(NextNumPos) + CarryFlag + Tiller \ 10
	DO WHILE Result%(NextNumPos) >= 10
		Result%(NextNumPos) = Result%(NextNumPos) - 10
		NextNumPos = NextNumPos + 1
		Result%(NextNumPos) = Result%(NextNumPos) + 1
	LOOP
NEXT QB
NEXT SF
END FUNCTION
*/

	// 'unsigned long long' makes the computation ugly with 32bit code, so going down to 'unsigned long':
	// 8:14:04.78
	// 8:25:46.22
	// vs
	// 1:41:43.84
	// 1:52:33.59
	// Or almost a minute ugliness!

	unsigned long SF, QB, Cycle, NextNumPos;
	unsigned long ResLength, CandLength, ErLength;
	unsigned char CarryFlag, TillerMostSignificantDigit, TillerLeastSignificantDigit;

	// 0..9 * 0..9
	unsigned char MSDarray[10][10] = {
		0,0,0,0,0,0,0,0,0,0, // 0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9,
		0,0,0,0,0,0,0,0,0,0, // 1x0, 1x1, 1x2, 1x3, 1x4, 1x5, 1x6, 1x7, 1x8, 1x9,
		0,0,0,0,0,1,1,1,1,1, // 2x0, 2x1, 2x2, 2x3, 2x4, 2x5, 2x6, 2x7, 2x8, 2x9, 
		0,0,0,0,1,1,1,2,2,2, // 3x0, 3x1, 3x2, 3x3, 3x4, 3x5, 3x6, 3x7, 3x8, 3x9, 
		0,0,0,1,1,2,2,2,3,3, // 4x0, 4x1, 4x2, 4x3, 4x4, 4x5, 4x6, 4x7, 4x8, 4x9, 
		0,0,1,1,2,2,3,3,4,4, // 5x0, 5x1, 5x2, 5x3, 5x4, 5x5, 5x6, 5x7, 5x8, 5x9, 
		0,0,1,1,2,3,3,4,4,5, // 6x0, 6x1, 6x2, 6x3, 6x4, 6x5, 6x6, 6x7, 6x8, 6x9, 
		0,0,1,2,2,3,4,4,5,6, // 7x0, 7x1, 7x2, 7x3, 7x4, 7x5, 7x6, 7x7, 7x8, 7x9, 
		0,0,1,2,3,4,4,5,6,7, // 8x0, 8x1, 8x2, 8x3, 8x4, 8x5, 8x6, 8x7, 8x8, 8x9, 
		0,0,1,2,3,4,5,6,7,8  // 9x0, 9x1, 9x2, 9x3, 9x4, 9x5, 9x6, 9x7, 9x8, 9x9
	};
	unsigned char LSDarray[10][10] = {
		0,0,0,0,0,0,0,0,0,0, // 0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9,
		0,1,2,3,4,5,6,7,8,9, // 1x0, 1x1, 1x2, 1x3, 1x4, 1x5, 1x6, 1x7, 1x8, 1x9,
		0,2,4,6,8,0,2,4,6,8, // 2x0, 2x1, 2x2, 2x3, 2x4, 2x5, 2x6, 2x7, 2x8, 2x9, 
		0,3,6,9,2,5,8,1,4,7, // 3x0, 3x1, 3x2, 3x3, 3x4, 3x5, 3x6, 3x7, 3x8, 3x9, 
		0,4,8,2,6,0,4,8,2,6, // 4x0, 4x1, 4x2, 4x3, 4x4, 4x5, 4x6, 4x7, 4x8, 4x9, 
		0,5,0,5,0,5,0,5,0,5, // 5x0, 5x1, 5x2, 5x3, 5x4, 5x5, 5x6, 5x7, 5x8, 5x9, 
		0,6,2,8,4,0,6,2,8,4, // 6x0, 6x1, 6x2, 6x3, 6x4, 6x5, 6x6, 6x7, 6x8, 6x9, 
		0,7,4,1,8,5,2,9,6,3, // 7x0, 7x1, 7x2, 7x3, 7x4, 7x5, 7x6, 7x7, 7x8, 7x9, 
		0,8,6,4,2,0,8,6,4,2, // 8x0, 8x1, 8x2, 8x3, 8x4, 8x5, 8x6, 8x7, 8x8, 8x9, 
		0,9,8,7,6,5,4,3,2,1  // 9x0, 9x1, 9x2, 9x3, 9x4, 9x5, 9x6, 9x7, 9x8, 9x9
	};
	// With above look-up tables (they replace all nasty *MULs, %MODs) we have a boost:
	// 1:41:43.84
	// 1:52:33.59 or 52*60+33-(41*60+43)= 650s
	// vs
	// 3:00:38.26
	// 3:08:11.38 or 08*60+11-(00*60+38)= 453s
	// Or 3+ minutes ugliness!
	
	CandLength =strlen(Multiplicand);
	ErLength =strlen(Multiplier);
	memset(Result, 0, (CandLength+ErLength)+1);

	if ( (ErLength == 1 && Multiplier[0] == 0+'0') || (CandLength == 1 && Multiplicand[0] == 0+'0') ) {
		Result[0] = 0+'0';
		return Result;
	}

	// In C the offset starts from 0 whereas in QuickBasic from 1, therefore '<' becomes '<='.
	for (SF=1; SF<=CandLength; SF++) {
	for (QB=1; QB<=ErLength; QB++) {
		CarryFlag = 0;
		Cycle = QB - 1 + SF;
		// In C the offset starts from 0 whereas in QuickBasic from 1, therefore '[SF]' becomes '[SF-1]'.
		// Here the strings are not reversed as in QB arrays, so [CandLength-SF+1] becomes [(CandLength-SF+1)-1].
		//Tiller = (Multiplicand[(CandLength-SF+1)-1]-'0') * (Multiplier[(ErLength-QB+1)-1]-'0');
		// LOOK-UP TABLE BOOST: Above COMMENTED line becomes next two:
		TillerLeastSignificantDigit = LSDarray[ Multiplicand[(CandLength-SF+1)-1]-'0' ][ Multiplier[(ErLength-QB+1)-1]-'0' ];
		TillerMostSignificantDigit = MSDarray[ Multiplicand[(CandLength-SF+1)-1]-'0' ][ Multiplier[(ErLength-QB+1)-1]-'0' ];
		//Result[Cycle-1] = (Tiller%10) + Result[Cycle-1];
		// LOOK-UP TABLE BOOST: Above COMMENTED line becomes next one:
		Result[Cycle-1] = TillerLeastSignificantDigit + Result[Cycle-1];
		if ( Result[Cycle-1] >= 10 ) {
			Result[Cycle-1] = Result[Cycle-1] - 10;
			CarryFlag = 1;
		}
		NextNumPos = Cycle + 1;
		//Result[NextNumPos-1] = Result[NextNumPos-1] + CarryFlag + (unsigned char)(Tiller/10);
		// LOOK-UP TABLE BOOST: Above COMMENTED line becomes next one:
		Result[NextNumPos-1] = Result[NextNumPos-1] + CarryFlag + TillerMostSignificantDigit;
		while (Result[NextNumPos-1] >= 10) {
			Result[NextNumPos-1] = Result[NextNumPos-1] - 10;
			NextNumPos = NextNumPos + 1;
			Result[NextNumPos-1] = Result[NextNumPos-1] + 1;
		}
	}
	}

	//Here we have the result (REVERSED) in ASCII codes i.e. '0' stands for ASCII 000.
	//ResLength is either (CandLength+ErLength) or (CandLength+ErLength-1).
	ResLength=(CandLength+ErLength);
	if (Result[ResLength-1] == 0x00) ResLength--;

	//The last thing to be done: the in-place reversal:
	for (SF=1; SF<=ResLength/2; SF++) {
		CarryFlag = Result[SF-1];
		Result[SF-1] = Result[(ResLength-SF+1)-1]+'0';
		Result[(ResLength-SF+1)-1] = CarryFlag+'0';
	}
	if (ResLength%2 != 0)
		Result[(ResLength/2+1)-1] = Result[(ResLength/2+1)-1]+'0';
	//Terminate it:
	Result[ResLength] = 0;

	return Result;
}

int main(int argc, char *argv[])
{
	signed int n, p, p31;
	unsigned char* pointerflushUNALIGN;
	unsigned char* Multiplicand;
	unsigned char* Multiplier;
	time_t t1, t2;
	unsigned long long DPS;
	FILE *fp_out;

	if (argc != 3 && argc != 4) {
		printf("MokujIN, Multiplication of INtegers, written by Kaze, 2012-Nov-07, revision 3+.\n");
		printf("Usage: MokujIN [Number Power [/turbo|/stats]]\n");
		printf("Note0: With no parameters given, interactive multiplication regime is on.\n");
		printf("Note1: Power is signed 32bit integer i.e. up to 2^31-1=2,147,483,647.\n");
		printf("Note2: Multiplicand or/and Multiplier cannot exceed %d digits.\n", memory_size);
		printf("Note3: '/turbo' regime is several times (only) faster.\n");
		printf("Note4: '/stats' is '/turbo' too, also dumps the result to 'MokujIN.txt'.\n");
		printf("Note5: When power is big enough (as in Example3) MokujINs (or the prosaic DPS) is an useful CPU clock pseudo-measure.\n");
		printf("Example1: D:\\_KAZE\\MokujIN>MokujIN.exe 18446744073709551616 2\n");
		printf("340282366920938463463374607431768211456\n");
		printf("Example2: D:\\_KAZE\\MokujIN>MokujIN.exe\n");
		printf("Multiplicand: 18446744073709551616\n");
		printf("Multiplier  : 18446744073709551616\n");
		printf("Result      : 340282366920938463463374607431768211456\n");
		printf("Example3: D:\\_KAZE\\MokujIN>MokujIN.exe 2 1048576 /stats\n");
		printf("...\n");
		printf("Multiplying performance for operands 39457 digits long: 77842742 MokujINs i.e. digits per second.\n");
		printf("Multiplying performance for operands 78914 digits long: 77842742 MokujINs i.e. digits per second.\n");
		printf("Multiplying performance for operands 157827 digits long: 77358266 MokujINs i.e. digits per second.\n");
		printf("Dumping the result to 'MokujIN.txt' ... OK\n");
	}

	pointerflushUNALIGN = (unsigned char*)malloc( memory_size*2+1 ); //+1 because we need sentinel
	if( pointerflushUNALIGN == NULL ) {
		printf("MokujIN: Needed memory allocation denied!\n");
		return( 1 );
	}
	Multiplicand = (unsigned char*)malloc( memory_size*2+1 ); //+1 because we need sentinel
	if( Multiplicand == NULL ) {
		printf("MokujIN: Needed memory allocation denied!\n");
		return( 1 );
	}
	Multiplier = (unsigned char*)malloc( memory_size*2+1 ); //+1 because we need sentinel
	if( Multiplier == NULL ) {
		printf("MokujIN: Needed memory allocation denied!\n");
		return( 1 );
	}

	if (argc == 3 || argc == 4) {
		p = atoi(argv[2]); //_atoi64(argv[2]);
		if (p < 2) {
			printf("MokujIN: Power should be 2 or greater!\n");
			free(pointerflushUNALIGN); free(Multiplicand); free(Multiplier);
			return( 2 );
		}
		memcpy(Multiplicand, argv[1], strlen(argv[1])); Multiplicand[strlen(argv[1])]=0; // Not assuming the allocated pool is ZEROed!
		memcpy(Multiplier, argv[1], strlen(argv[1])); Multiplier[strlen(argv[1])]=0;
		if (argc == 4) {
			p31 = 0;
			while (p>>1) {
				p31++;
				p = p>>1;
			}
			p = atoi(argv[2]);
			p = p - (1<<p31);
			while (p31--) {
				(void) time(&t1);
				MUL(pointerflushUNALIGN, Multiplicand, Multiplier);
				(void) time(&t2);
				if (t2 <= t1) {t2 = t1; t2++;}
				if ( strcmp("/stats\0",argv[3]) == 0 ) {
					DPS = (unsigned long long)strlen(Multiplier);
					DPS *= DPS;
					DPS = DPS/(unsigned long long)(t2-t1);
					printf("Multiplying performance for operands %lu digits long: %lu MokujINs i.e. digits per second.\n", strlen(Multiplier), DPS);
				}
				memcpy(Multiplicand, pointerflushUNALIGN, strlen(pointerflushUNALIGN)); Multiplicand[strlen(pointerflushUNALIGN)]=0;
				memcpy(Multiplier, pointerflushUNALIGN, strlen(pointerflushUNALIGN)); Multiplier[strlen(pointerflushUNALIGN)]=0;
			}
			memcpy(Multiplier, argv[1], strlen(argv[1])); Multiplier[strlen(argv[1])]=0;
			while (p--) {
				MUL(pointerflushUNALIGN, Multiplicand, Multiplier);
				memcpy(Multiplicand, pointerflushUNALIGN, strlen(pointerflushUNALIGN)); Multiplicand[strlen(pointerflushUNALIGN)]=0;
			}
		} else {
			while (p-->1) {
				MUL(pointerflushUNALIGN, Multiplicand, Multiplier);
				memcpy(Multiplicand, pointerflushUNALIGN, strlen(pointerflushUNALIGN)); Multiplicand[strlen(pointerflushUNALIGN)]=0;
			}
		}
		if ( argc == 4 && strcmp("/stats\0",argv[3]) == 0 ) {
			if( ( fp_out = fopen( "MokujIN.txt", "wb+" ) ) == NULL )
			{ printf( "MokujIN: Can't create file 'MokujIN.txt'.\n" ); return( 4 ); }
			printf("Dumping the result to 'MokujIN.txt' ... ");
			fprintf( fp_out, "%s\r\n", Multiplicand );
			fclose(fp_out);
			printf("OK\n");
		} else
			printf("%s\n", Multiplicand);
		free(pointerflushUNALIGN); free(Multiplicand); free(Multiplier);
		return 0;
	} else if (argc == 1) {
		printf("\nMultiplicand: ");
		scanf("%s", Multiplicand);
		printf("Multiplier  : ");
		scanf("%s", Multiplier);
		printf("Result      : ");
		printf("%s\n", MUL(pointerflushUNALIGN, Multiplicand, Multiplier));
		free(pointerflushUNALIGN); free(Multiplicand); free(Multiplier);
		return 0;
	} else {
		free(pointerflushUNALIGN); free(Multiplicand); free(Multiplier);
		return 3;
	}
}

// The following assembly/code dump is the main loop of MokujIN (namely: 'MUL' function):
// It is 32b-290+6 = 161 bytes long.
// D:\_KAZE\MokujIN>cl /Ox MokujIN.c /FAcs
// Microsoft (R) 32-bit C/C++ Optimizing Compiler Version 16.00.30319.01 for 80x86
/*
$LL14@MUL:

; 122  : 	for (QB=1; QB<=ErLength; QB++) {

  00290	83 ff 01	 cmp	 edi, 1
  00293	0f 82 87 00 00
	00		 jb	 $LN13@MUL
  00299	8b 54 24 24	 mov	 edx, DWORD PTR _ErLength$[esp+256]
  0029d	8d 45 ff	 lea	 eax, DWORD PTR [ebp-1]
  002a0	8d 34 01	 lea	 esi, DWORD PTR [ecx+eax]
  002a3	8b 4c 24 1c	 mov	 ecx, DWORD PTR _Multiplier$GSCopy$[esp+256]
  002a7	8d 7c 39 ff	 lea	 edi, DWORD PTR [ecx+edi-1]
  002ab	8b cd		 mov	 ecx, ebp
  002ad	2b c8		 sub	 ecx, eax
  002af	89 4c 24 30	 mov	 DWORD PTR tv1469[esp+256], ecx
  002b3	89 54 24 28	 mov	 DWORD PTR tv771[esp+256], edx
  002b7	eb 07 8d a4 24
	00 00 00 00	 npad	 9
$LL30@MUL:

; 123  : 		CarryFlag = 0;
; 124  : 		Cycle = QB - 1 + SF;
; 125  : 		// In C the offset starts from 0 whereas in QuickBasic from 1, therefore '[SF]' becomes '[SF-1]'.
; 126  : 		// Here the strings are not reversed as in QB arrays, so [CandLength-SF+1] becomes [(CandLength-SF+1)-1].
; 127  : 		//Tiller = (Multiplicand[(CandLength-SF+1)-1]-'0') * (Multiplier[(ErLength-QB+1)-1]-'0');
; 128  : 		// LOOK-UP TABLE BOOST: Above COMMENTED line becomes next two:
; 129  : 		TillerLeastSignificantDigit = LSDarray[ Multiplicand[(CandLength-SF+1)-1]-'0' ][ Multiplier[(ErLength-QB+1)-1]-'0' ];
; 130  : 		TillerMostSignificantDigit = MSDarray[ Multiplicand[(CandLength-SF+1)-1]-'0' ][ Multiplier[(ErLength-QB+1)-1]-'0' ];

  002c0	8b 44 24 14	 mov	 eax, DWORD PTR tv1486[esp+256]
  002c4	0f b6 00	 movzx	 eax, BYTE PTR [eax]
  002c7	8d 0c 80	 lea	 ecx, DWORD PTR [eax+eax*4]
  002ca	0f b6 07	 movzx	 eax, BYTE PTR [edi]
  002cd	8d 04 48	 lea	 eax, DWORD PTR [eax+ecx*2]
  002d0	8a 8c 04 24 fe
	ff ff		 mov	 cl, BYTE PTR _MSDarray$[esp+eax-272]

; 131  : 		//Result[Cycle-1] = (Tiller%10) + Result[Cycle-1];
; 132  : 		// LOOK-UP TABLE BOOST: Above COMMENTED line becomes next one:
; 133  : 		Result[Cycle-1] = TillerLeastSignificantDigit + Result[Cycle-1];

  002d7	8a 84 04 88 fe
	ff ff		 mov	 al, BYTE PTR _LSDarray$[esp+eax-272]
  002de	02 06		 add	 al, BYTE PTR [esi]
  002e0	32 d2		 xor	 dl, dl
  002e2	88 06		 mov	 BYTE PTR [esi], al

; 134  : 		if ( Result[Cycle-1] >= 10 ) {

  002e4	3c 0a		 cmp	 al, 10			; 0000000aH
  002e6	72 06		 jb	 SHORT $LN8@MUL

; 135  : 			Result[Cycle-1] = Result[Cycle-1] - 10;

  002e8	2c 0a		 sub	 al, 10			; 0000000aH
  002ea	88 06		 mov	 BYTE PTR [esi], al

; 136  : 			CarryFlag = 1;

  002ec	b2 01		 mov	 dl, 1
$LN8@MUL:

; 139  : 		//Result[NextNumPos-1] = Result[NextNumPos-1] + CarryFlag + (unsigned char)(Tiller/10);
; 140  : 		// LOOK-UP TABLE BOOST: Above COMMENTED line becomes next one:
; 141  : 		Result[NextNumPos-1] = Result[NextNumPos-1] + CarryFlag + TillerMostSignificantDigit;

  002ee	02 ca		 add	 cl, dl
  002f0	00 4e 01	 add	 BYTE PTR [esi+1], cl

; 142  : 		while (Result[NextNumPos-1] >= 10) {

  002f3	80 7e 01 0a	 cmp	 BYTE PTR [esi+1], 10	; 0000000aH
  002f7	72 13		 jb	 SHORT $LN10@MUL

; 137  : 		}
; 138  : 		NextNumPos = Cycle + 1;

  002f9	8b 44 24 30	 mov	 eax, DWORD PTR tv1469[esp+256]
  002fd	03 c6		 add	 eax, esi
  002ff	90		 npad	 1
$LL7@MUL:

; 143  : 			Result[NextNumPos-1] = Result[NextNumPos-1] - 10;

  00300	80 00 f6	 add	 BYTE PTR [eax], 246	; 000000f6H

; 144  : 			NextNumPos = NextNumPos + 1;
; 145  : 			Result[NextNumPos-1] = Result[NextNumPos-1] + 1;

  00303	fe 40 01	 inc	 BYTE PTR [eax+1]
  00306	40		 inc	 eax
  00307	80 38 0a	 cmp	 BYTE PTR [eax], 10	; 0000000aH
  0030a	73 f4		 jae	 SHORT $LL7@MUL
$LN10@MUL:

; 139  : 		//Result[NextNumPos-1] = Result[NextNumPos-1] + CarryFlag + (unsigned char)(Tiller/10);
; 140  : 		// LOOK-UP TABLE BOOST: Above COMMENTED line becomes next one:
; 141  : 		Result[NextNumPos-1] = Result[NextNumPos-1] + CarryFlag + TillerMostSignificantDigit;

  0030c	46		 inc	 esi
  0030d	4f		 dec	 edi
  0030e	ff 4c 24 28	 dec	 DWORD PTR tv771[esp+256]
  00312	75 ac		 jne	 SHORT $LL30@MUL

; 122  : 	for (QB=1; QB<=ErLength; QB++) {

  00314	8b 4c 24 20	 mov	 ecx, DWORD PTR _SF$[esp+256]
  00318	8b 7c 24 24	 mov	 edi, DWORD PTR _ErLength$[esp+256]
  0031c	8b 44 24 10	 mov	 eax, DWORD PTR _CandLength$[esp+256]
$LN13@MUL:

; 118  : 	}
; 119  : 
; 120  : 	// In C the offset starts from 0 whereas in QuickBasic from 1, therefore '<' becomes '<='.
; 121  : 	for (SF=1; SF<=CandLength; SF++) {

  00320	ff 4c 24 14	 dec	 DWORD PTR tv1486[esp+256]
  00324	41		 inc	 ecx
  00325	89 4c 24 20	 mov	 DWORD PTR _SF$[esp+256], ecx
  00329	3b c8		 cmp	 ecx, eax
  0032b	0f 86 5f ff ff
	ff		 jbe	 $LL14@MUL
$LN12@MUL:
*/


// The next functions were written by Kaze in 1990.
/*
DECLARE SUB DelNull (Result%(), Tiller$, DimLength%)
DECLARE SUB DelLeftNull (Result%(), Tiller$, StrLength%, DimLength%, PointPos%)
DECLARE SUB DelRightNull (Result%(), Tiller$, StrLength%, PointPos%)
DECLARE SUB Convert (Array%(), Number$, Length%, Flag%, PointPos%)
DECLARE FUNCTION FindIntegerPart$ (Dividend$, Divisor$)
DECLARE FUNCTION DIV$ (Dividend$, Divisor$, Exactness%)
DECLARE FUNCTION ADD$ (FirstAddend$, SecondAddend$)
DECLARE FUNCTION SUB$ (Minuend$, Subtrahend$)
DECLARE FUNCTION MUL$ (Multiplicand$, Multiplier$)
DECLARE FUNCTION Compare% (Firum$, Secum$)
DECLARE FUNCTION ResultFilter$ (Result%(), Code$, DimLength%, PointPos%, Sign%)

INPUT "Number: ", a$
INPUT "Power: ", b$

IF VAL(b$) < 2 THEN PRINT "Power should be 2 or greater!": END
c$ = a$
FOR i = 2 TO VAL(b$)
c$ = MUL$(c$, a$)
NEXT
PRINT c$


DEFINT A-Z
FUNCTION ADD$ (FirstAddend$, SecondAddend$)
FiradLength = LEN(FirstAddend$)
SecadLength = LEN(SecondAddend$)
FirstSign$ = MID$(FirstAddend$, 1, 1)
SecondSign$ = MID$(SecondAddend$, 1, 1)
IF FirstSign$ <> "-" AND SecondSign$ = "-" THEN
Mediator$ = RIGHT$(SecondAddend$, SecadLength - 1)
ADD$ = SUB$(FirstAddend$, Mediator$)
EXIT FUNCTION
END IF
IF FirstSign$ = "-" AND SecondSign$ <> "-" THEN
Mediator$ = RIGHT$(FirstAddend$, FiradLength - 1)
ADD$ = SUB$(SecondAddend$, Mediator$)
EXIT FUNCTION
END IF
IF FirstSign$ = "-" AND SecondSign$ = "-" THEN
Sign = 1
FiradLength = FiradLength - 1
SecadLength = SecadLength - 1
FirstAddend$ = RIGHT$(FirstAddend$, FiradLength)
SecondAddend$ = RIGHT$(SecondAddend$, SecadLength)
END IF
REDIM Firad%(1 TO FiradLength)
REDIM Secad%(1 TO SecadLength)
CALL Convert(Firad%(), FirstAddend$, FiradLength, FiradFlag, FiradPointPos)
FiradTrueLength = FiradLength - FiradPointPos
CALL Convert(Secad%(), SecondAddend$, SecadLength, SecadFlag, SecadPointPos)
SecadTrueLength = SecadLength - SecadPointPos
IF SecadTrueLength > FiradTrueLength THEN
BigLength = SecadTrueLength
ELSE
BigLength = FiradTrueLength
END IF
IF FiradPointPos > SecadPointPos THEN
BigPointPos = FiradPointPos
TrueBigPointPos = BigPointPos - FiradFlag
SubSecad = SecadPointPos - SecadFlag - TrueBigPointPos
ELSE
BigPointPos = SecadPointPos
TrueBigPointPos = BigPointPos - SecadFlag
SubFirad = FiradPointPos - FiradFlag - TrueBigPointPos
END IF
DimSum = BigLength + TrueBigPointPos + 1
REDIM Result%(1 TO DimSum)
FOR SF = 1 TO BigLength + BigPointPos
FiradDim = 0
SecadDim = 0
SubFirad = SubFirad + 1
SubSecad = SubSecad + 1
IF SubFirad > 0 AND SubFirad <= FiradLength THEN FiradDim = Firad%(SubFirad)
IF SubSecad > 0 AND SubSecad <= SecadLength THEN SecadDim = Secad%(SubSecad)
Result%(SF) = FiradDim + SecadDim + Result%(SF)
IF Result%(SF) >= 10 THEN
Result%(SF + 1) = 1
Result%(SF) = Result(SF) - 10
END IF
NEXT SF
ADD$ = ResultFilter$(Result%(), "ADD", DimSum, BigPointPos, Sign)
END FUNCTION

FUNCTION Compare% (Firum$, Secum$)
FirLeftPart = INSTR(Firum$, ".")
SecLeftPart = INSTR(Secum$, ".")
IF FirLeftPart = 0 THEN
FirLeftPart = LEN(Firum$)
ELSE
FirLeftPart = FirLeftPart - 1
FirRightPart = LEN(Firum$) - FirLeftPart
END IF
IF SecLeftPart = 0 THEN
SecLeftPart = LEN(Secum$)
ELSE
SecLeftPart = SecLeftPart - 1
SecRightPart = LEN(Secum$) - SecLeftPart
END IF
IF SecRightPart > FirRightPart THEN
MainRightPart = FirRightPart
ELSE
MainRightPart = SecRightPart
END IF
IF SecLeftPart > FirLeftPart THEN
CompareCode = -1
MainLeftPart = SecLeftPart
ELSE
CompareCode = 1
MainLeftPart = FirLeftPart
END IF
IF SecLeftPart = FirLeftPart THEN
CompareCode = 0
DO
Counter = Counter + 1
FirumPart = VAL(MID$(Firum$, Counter, 1))
SecumPart = VAL(MID$(Secum$, Counter, 1))
IF FirumPart > SecumPart THEN CompareCode = 1
IF FirumPart < SecumPart THEN CompareCode = -1
LOOP WHILE Counter < MainLeftPart AND FirumPart = SecumPart
IF CompareCode = 0 THEN
MainLength = MainLeftPart + MainRightPart
IF MainRightPart > 0 THEN
Counter = Counter + 1
DO
Counter = Counter + 1
FirumPart = VAL(MID$(Firum$, Counter, 1))
SecumPart = VAL(MID$(Secum$, Counter, 1))
IF FirumPart > SecumPart THEN CycleFlag = 1: CompareCode = 1
IF FirumPart < SecumPart THEN CycleFlag = 1: CompareCode = -1
LOOP WHILE Counter < MainLength AND FirumPart = SecumPart
END IF
IF Counter = MainLength AND CycleFlag = 0 THEN
IF FirRightPart < SecRightPart THEN CompareCode = -1
IF FirRightPart > SecRightPart THEN CompareCode = 1
END IF
END IF
END IF
Compare% = CompareCode
END FUNCTION

SUB Convert (Array%(), Number$, Length, Flag, PointPos)
FOR Counter = 1 TO Length
Shoot$ = MID$(Number$, Length - Counter + 1, 1)
IF Shoot$ = "." THEN
Flag = 1
PointPos = Counter
Counter = Counter + 1
Shoot$ = MID$(Number$, Length - Counter + 1, 1)
END IF
Array%(Counter - Flag) = VAL(Shoot$)
NEXT Counter
END SUB

SUB DelLeftNull (Result%(), Tiller$, StrLength, DimLength, PointPos)
Counter = DimLength
DeviPart = DimLength + 1 - StrLength
DO WHILE PointPos + 1 <= Counter - DeviPart AND Result%(Counter) = 0
StrLength = StrLength - 1
Tiller$ = RIGHT$(Tiller$, StrLength)
Counter = Counter - 1
LOOP
END SUB

SUB DelNull (Result%(), Tiller$, DimLength)
Counter = DimLength
DO WHILE Result%(Counter) = 0 AND Counter >= 2
Tiller$ = RIGHT$(Tiller$, Counter - 1)
Counter = Counter - 1
LOOP
END SUB

SUB DelRightNull (Result%(), Tiller$, StrLength, PointPos)
Counter = 1
NewPointPos = PointPos
DO WHILE PointPos - 2 >= Counter AND Result%(Counter) = 0
StrLength = StrLength - 1
Tiller$ = LEFT$(Tiller$, StrLength)
Counter = Counter + 1
NewPointPos = NewPointPos - 1
LOOP
PointPos = NewPointPos
END SUB

FUNCTION DIV$ (Dividend$, Divisor$, Exactness)
DendSign$ = MID$(Dividend$, 1, 1)
SorSign$ = MID$(Divisor$, 1, 1)
IF DendSign$ = "-" AND SorSign$ <> "-" THEN
Sign = 1
Dividend$ = RIGHT$(Dividend$, LEN(Dividend$) - 1)
END IF
IF DendSign$ <> "-" AND SorSign$ = "-" THEN
Sign = 1
Divisor$ = RIGHT$(Divisor$, LEN(Divisor$) - 1)
END IF
IF DendSign$ = "-" AND SorSign$ = "-" THEN
Divisor$ = RIGHT$(Divisor$, LEN(Divisor$) - 1)
Dividend$ = RIGHT$(Dividend$, LEN(Dividend$) - 1)
END IF
DendPointPos = INSTR(Dividend$, ".")
SorPointPos = INSTR(Divisor$, ".")
IF DendPointPos <> 0 THEN
DendRightPart = LEN(Dividend$) - DendPointPos
DendRightPart$ = RIGHT$(Dividend$, DendRightPart)
Dividend$ = LEFT$(Dividend$, DendPointPos - 1) + DendRightPart$
END IF
IF SorPointPos <> 0 THEN
SorRightPart = LEN(Divisor$) - SorPointPos
SorRightPart$ = RIGHT$(Divisor$, SorRightPart)
Divisor$ = LEFT$(Divisor$, SorPointPos - 1) + SorRightPart$
END IF
DO WHILE MID$(Dividend$, 1, 1) = "0" AND LEN(Dividend$) <> 1
Dividend$ = RIGHT$(Dividend$, LEN(Dividend$) - 1)
LOOP
DO WHILE MID$(Divisor$, 1, 1) = "0" AND LEN(Divisor$) <> 1
Divisor$ = RIGHT$(Divisor$, LEN(Divisor$) - 1)
LOOP
IF Divisor$ = "0" THEN ERROR 11
DendLength = LEN(Dividend$)
SorLength = LEN(Divisor$)
IF DendLength <= SorLength THEN
Tiller$ = Dividend$
Mediator$ = FindIntegerPart$(Dividend$, Divisor$)
IntegerPart$ = Mediator$
ELSE
Pointer = SorLength
Tiller$ = MID$(Dividend$, 1, Pointer)
IF FindIntegerPart$(Tiller$, Divisor$) = "0" THEN
Pointer = SorLength + 1
Tiller$ = MID$(Dividend$, 1, Pointer)
END IF
DO WHILE Pointer < DendLength
Mediator$ = FindIntegerPart$(Tiller$, Divisor$)
IntegerPart$ = IntegerPart$ + Mediator$
Pointer = Pointer + 1
Shoot$ = MUL$(Mediator$, Divisor$)
Deviation$ = SUB$(Tiller$, Shoot$)
IF Deviation$ = "0" THEN
Tiller$ = MID$(Dividend$, Pointer, 1)
ELSE
Tiller$ = Deviation$ + MID$(Dividend$, Pointer, 1)
END IF
LOOP
Mediator$ = FindIntegerPart$(Tiller$, Divisor$)
IntegerPart$ = IntegerPart$ + Mediator$
END IF
Result$ = IntegerPart$
RelativePosition = DendRightPart - SorRightPart
MiddlePosition = LEN(IntegerPart$) - RelativePosition
CycleLength = Exactness - RelativePosition
IF CycleLength > 0 THEN
FOR i = 1 TO CycleLength
Shoot$ = MUL$(Mediator$, Divisor$)
Deviation$ = SUB$(Tiller$, Shoot$)
IF Deviation$ = "0" THEN Tiller$ = "0" ELSE Tiller$ = Deviation$ + "0"
Mediator$ = FindIntegerPart$(Tiller$, Divisor$)
FractionalPart$ = FractionalPart$ + Mediator$
NEXT i
Result$ = Result$ + FractionalPart$
END IF
IF MiddlePosition > 0 THEN
LeftPart$ = LEFT$(Result$, MiddlePosition)
IF Exactness = 0 THEN
Result$ = LeftPart$
ELSE
RightPart$ = RIGHT$(Result$, LEN(Result$) - MiddlePosition)
Result$ = LeftPart$ + "." + RightPart$
END IF
IF DendLength <= SorLength THEN
        'Below lines for what captain.
        'NullNumber = SorLength - DendLength + 1 'Bug appears here
        '                                        'because +1 or +0 must.
        'IF MiddlePosition > NullNumber THEN
        'DelLength = NullNumber
        'ELSEIF MiddlePosition = NullNumber THEN
        'DelLength = NullNumber - 1
        'ELSEIF MiddlePosition > 1 THEN
        'DelLength = MiddlePosition - 1
        'END IF
        'Result$ = RIGHT$(Result$, LEN(Result$) - DelLength)
        DO WHILE LEN(Result$) >= 2
        IF LEFT$(Result$, 1) = "0" AND MID$(Result$, 2, 1) <> "." THEN
        Result$ = RIGHT$(Result$, LEN(Result$) - 1)
        ELSE
        EXIT DO
        END IF
        LOOP
END IF
ELSE
IF MiddlePosition = 0 THEN
Result$ = "0." + Result$
ELSE
Result$ = "0." + STRING$(ABS(MiddlePosition), "0") + Result$
END IF
IF Exactness = 0 THEN
Result$ = "0"
ELSEIF CycleLength < 0 THEN
Result$ = LEFT$(Result$, 2 + Exactness)
END IF
END IF
IF Sign = 1 THEN DIV$ = "-" + Result$ ELSE DIV$ = Result$
END FUNCTION

FUNCTION FindIntegerPart$ (Dividend$, Divisor$)
DO
Counter = Counter + 1
Digit$ = LTRIM$(STR$(Counter))
Tiller$ = MUL$(Digit$, Divisor$)
CompareCode = Compare%(Tiller$, Dividend$)
LOOP WHILE Counter <= 9 AND CompareCode = -1
FindIntegerPart$ = LTRIM$(STR$(Counter - CompareCode))
END FUNCTION

FUNCTION MUL$ (Multiplicand$, Multiplier$)
CandSign$ = MID$(Multiplicand$, 1, 1)
ErSign$ = MID$(Multiplier$, 1, 1)
CandLength = LEN(Multiplicand$)
ErLength = LEN(Multiplier$)
IF CandSign$ = "-" AND ErSign$ <> "-" THEN
Sign = 1
CandLength = CandLength - 1
Multiplicand$ = RIGHT$(Multiplicand$, CandLength)
END IF
IF CandSign$ <> "-" AND ErSign$ = "-" THEN
Sign = 1
ErLength = ErLength - 1
Multiplier$ = RIGHT$(Multiplier$, ErLength)
END IF
IF CandSign$ = "-" AND ErSign$ = "-" THEN
ErLength = ErLength - 1
CandLength = CandLength - 1
Multiplier$ = RIGHT$(Multiplier$, ErLength)
Multiplicand$ = RIGHT$(Multiplicand$, CandLength)
END IF
REDIM Cand%(1 TO CandLength)
REDIM Er%(1 TO ErLength)
CALL Convert(Cand%(), Multiplicand$, CandLength, CandFlag, CandPointPos)
CALL Convert(Er%(), Multiplier$, ErLength, ErFlag, ErPointPos)
DimSum = CandLength + ErLength - CandFlag - ErFlag
PointPos = CandPointPos + ErPointPos - CandFlag - ErFlag
REDIM Result%(1 TO DimSum)
FOR SF = 1 TO CandLength - CandFlag
FOR QB = 1 TO ErLength - ErFlag
CarryFlag = 0
Cycle = QB - 1 + SF
Tiller = Cand%(SF) * Er%(QB)
Result%(Cycle) = Tiller MOD 10 + Result%(Cycle)
IF Result%(Cycle) >= 10 THEN
Result%(Cycle) = Result%(Cycle) - 10
CarryFlag = 1
END IF
NextNumPos = Cycle + 1
Result%(NextNumPos) = Result%(NextNumPos) + CarryFlag + Tiller \ 10
DO WHILE Result%(NextNumPos) >= 10
Result%(NextNumPos) = Result%(NextNumPos) - 10
NextNumPos = NextNumPos + 1
Result%(NextNumPos) = Result%(NextNumPos) + 1
LOOP
NEXT QB
NEXT SF
MUL$ = ResultFilter$(Result%(), "MUL", DimSum, PointPos, Sign)
END FUNCTION

FUNCTION ResultFilter$ (Result%(), Code$, DimLength, PointPos, Sign)
SELECT CASE Code$
CASE "ADD"
FOR i = 1 TO DimLength - 1
IF i = PointPos THEN Tiller$ = "." + Tiller$
Tiller$ = LTRIM$(STR$(Result%(i))) + Tiller$
NEXT i
IF Result%(DimLength) <> 0 THEN
Tiller$ = LTRIM$(STR$(Result%(DimLength))) + Tiller$
END IF
IF PointPos <> 0 THEN
StrLength = LEN(Tiller$)
CALL DelRightNull(Result%(), Tiller$, StrLength, PointPos)
END IF
IF Sign = 1 THEN Tiller$ = "-" + Tiller$
CASE "SUB"
FOR i = 1 TO DimLength
IF i = PointPos THEN Tiller$ = "." + Tiller$
Tiller$ = LTRIM$(STR$(Result%(i))) + Tiller$
NEXT i
IF PointPos <> 0 THEN
StrLength = LEN(Tiller$)
CALL DelRightNull(Result%(), Tiller$, StrLength, PointPos)
CALL DelLeftNull(Result%(), Tiller$, StrLength, DimLength, PointPos)
ELSE
CALL DelNull(Result%(), Tiller$, DimLength)
END IF
IF Sign = -1 THEN Tiller$ = "-" + Tiller$
CASE "MUL"
FOR i = 1 TO DimLength
Tiller$ = LTRIM$(STR$(Result%(i))) + Tiller$
IF i = PointPos THEN Tiller$ = "." + Tiller$
NEXT i
IF Result%(DimLength) = 0 THEN
IF PointPos = 0 THEN
Tiller$ = RIGHT$(Tiller$, DimLength - 1)
IF MID$(Tiller$, 1, 1) = "0" THEN Tiller$ = "0"
ELSE
PointPos = PointPos + 1
Tiller$ = RIGHT$(Tiller$, DimLength)
StrLength = LEN(Tiller$)
TrueDimLength = StrLength - 1
CALL DelRightNull(Result%(), Tiller$, StrLength, PointPos)
CALL DelLeftNull(Result%(), Tiller$, StrLength, TrueDimLength, PointPos)
END IF
END IF
IF Sign = 1 THEN Tiller$ = "-" + Tiller$
END SELECT
IF RIGHT$(Tiller$, 2) = ".0" THEN Tiller$ = LEFT$(Tiller$, LEN(Tiller$) - 2)
IF Tiller$ = "-0" THEN Tiller$ = "0"
ResultFilter$ = Tiller$
END FUNCTION

FUNCTION SUB$ (Minuend$, Subtrahend$)
MinuSign$ = MID$(Minuend$, 1, 1)
SubtraSign$ = MID$(Subtrahend$, 1, 1)
MinuLength = LEN(Minuend$)
SubtraLength = LEN(Subtrahend$)
IF MinuSign$ = "-" AND SubtraSign$ <> "-" THEN
Mediator$ = RIGHT$(Minuend$, MinuLength - 1)
SUB$ = "-" + ADD$(Mediator$, Subtrahend$)
EXIT FUNCTION
END IF
IF MinuSign$ <> "-" AND SubtraSign$ = "-" THEN
Mediator$ = RIGHT$(Subtrahend$, SubtraLength - 1)
SUB$ = ADD$(Minuend$, Mediator$)
EXIT FUNCTION
END IF
IF MinuSign$ = "-" AND SubtraSign$ = "-" THEN
SWAP Minuend$, Subtrahend$
MinuLength = MinuLength - 1
SubtraLength = SubtraLength - 1
Minuend$ = RIGHT$(Minuend$, MinuLength)
Subtrahend$ = RIGHT$(Subtrahend$, SubtraLength)
END IF
REDIM Minu%(1 TO MinuLength)
REDIM Subtra%(1 TO SubtraLength)
CALL Convert(Minu%(), Minuend$, MinuLength, MinuFlag, MinuPointPos)
MinuTrueLength = MinuLength - MinuPointPos
CALL Convert(Subtra%(), Subtrahend$, SubtraLength, SubtraFlag, SubtraPointPos)
SubtraTrueLength = SubtraLength - SubtraPointPos
IF SubtraTrueLength > MinuTrueLength THEN
MainTrueLength = SubtraTrueLength
ELSE
MainTrueLength = MinuTrueLength
END IF
CompareCode = Compare%(Minuend$, Subtrahend$)
IF CompareCode = -1 THEN Sign = -1 ELSE Sign = 1
IF MinuPointPos < SubtraPointPos THEN
BigPointPos = SubtraPointPos
TrueBigPointPos = BigPointPos - SubtraFlag
MinuSub = MinuPointPos - MinuFlag - TrueBigPointPos
ELSE
BigPointPos = MinuPointPos
TrueBigPointPos = BigPointPos - MinuFlag
SubtraSub = SubtraPointPos - SubtraFlag - TrueBigPointPos
END IF
DimSum = MainTrueLength + TrueBigPointPos
REDIM Result%(1 TO DimSum)
FOR SF = 1 TO DimSum
MinuDim = 0
SubtraDim = 0
MinuSub = MinuSub + 1
SubtraSub = SubtraSub + 1
IF MinuSub > 0 AND MinuSub <= MinuLength - MinuFlag THEN
MinuDim = Minu%(MinuSub)
END IF
IF SubtraSub > 0 AND SubtraSub <= SubtraLength - SubtraFlag THEN
SubtraDim = Subtra%(SubtraSub)
END IF
IF Sign = 1 THEN
MinuDim = MinuDim + SubFlag
ELSE
SubtraDim = SubtraDim + SubFlag
END IF
SubSum = Sign * (MinuDim - SubtraDim)
SubFlag = 0
IF Sign = 1 THEN
IF MinuDim >= SubtraDim THEN
Result%(SF) = SubSum
ELSE
IF SubSum < 0 THEN SubFlag = -1
Result%(SF) = SubSum + 10
END IF
ELSE
IF MinuDim > SubtraDim THEN
IF SubSum < 0 THEN SubFlag = -1
Result%(SF) = SubSum + 10
ELSE
Result%(SF) = SubSum
END IF
END IF
NEXT SF
SUB$ = ResultFilter$(Result%(), "SUB", DimSum, BigPointPos, Sign)
END FUNCTION
*/
